/*
 * Created on 11/10/05 20:17:06
 */
package edu.wpi.ejutr.file_operations;

import java.io.*;
import java.util.*;
import edu.wpi.ejutr.logger.*;
import edu.wpi.ejutr.debug.DebugDialog;

import org.eclipse.core.resources.*;
import org.eclipse.jdt.core.*;

import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;

/**
 * EJUTRFileWriter is responsible for taking an EBOBActionLogger and generating
 * a JUnit method in an existing Java test case file can be executed utilizing the jUnit plugin.
 * 
 * @author Ben Mohlenhoff - <a href="mailto:bigben@wpi.edu">bigben@wpi.edu</a>
 */
public class EJUTRFileWriter {
	
	/**
	 * The file to write to.
	 */
	private IFile testCaseFile;
		
	/**
	 * Constructs a new file writer.
	 * @param dest the destination file to write to.
	 */
	public EJUTRFileWriter(IFile file) {
		testCaseFile = file;
		
	}
		
	/**
	 * Dumps the contents of the EBOBActionLogger to the file specified when this
	 * EJUTRFileWriter was created.
	 */
	public void dumpToNewFile() {
		
		DebugDialog.show("dumpToNewFile called");
		FileOutputStream output;
		EBOBActionLogger log = EBOBActionLogger.getInstance();
		try {
			output = new FileOutputStream((File)testCaseFile);
			
			Iterator it = log.iterator();
			
			while (it.hasNext()) {
				EJUTRLogEntry entry = (EJUTRLogEntry)it.next();				
				output.write(entry.to_String().getBytes());
				
				DebugDialog.show("Writing To Disk: " + entry.to_String());				
			}					
			
			output.close(); // if you don't close it, nothing will be dumped to the physical hard disk
			
		} catch (FileNotFoundException fnfe) {
			// probably should display some dialog if the filename is no good, although
			// quite honestly it should be detected earlier.
		} catch (IOException ioe) {
			// probably should display a message here in case file output fails
		}
			
	}
	
	/**
	 * Dumps the contents of the EBOBActionLogger to the file specified when this
	 * EJUTRFileWriter was created, and places them into a separate method specified
	 * here as a parameter.  The method will be over written if it alreasy exist
	 * @param methodName the test method name to create.
	 */
	public void dumpTo(String methodName) {
		
		if(testCaseFile.exists() == false){
			MessageDialog.openError(new Shell(), "File Missing",
			"The file you were recording to no longer exits");;
			return;
		}
		
		EBOBActionLogger log = EBOBActionLogger.getInstance();
		Iterator it = log.iterator();
		//build the test method
		String methodContents = "public void " + methodName + "() {\n";
		while (it.hasNext()) {
			EJUTRLogEntry entry = (EJUTRLogEntry)it.next();	
			methodContents = methodContents + entry.to_String();
			DebugDialog.printToConsole(entry.to_String());
		}
		methodContents = methodContents + "}\n";
		//System.out.println(methodContents);
		ICompilationUnit myCU = (ICompilationUnit) JavaCore.create(testCaseFile);
		try {
			// add Assert import, if its not there already
			myCU.createImport("junit.framework.Assert", null, 0, null); // this call won't add it if it already exists.
						
			IType[] types = myCU.getTypes();
			types[0].createMethod(methodContents,null,true,null);
		} catch (JavaModelException e) {
			e.printStackTrace();
		}
	}
	
}
