package edu.wpi.ejutr.logger;

/**
 * Interface representing entries into the log, which are then
 * recorded.  At some later point, the to_String() method will be called
 * on objects that implement this interface, and the results will be placed
 * directly into the JUnit test method that is generated.
 * 
 * @author bigben - <a href="mailto:bigben@wpi.edu">bigben@wpi.edu</a>
 *
 */
public interface EJUTRLogEntry {
	
	/**
	 * Returns a string representation of the log entry.  Ideally this will be 
	 * placed directly into a file output stream.  Therefore, the responsibility
	 * of each class that implements this interface is to ensure that this method
	 * returns a string utilizing proper Java/JUnit syntax.
	 * 
	 * @return String representation of log entry
	 */
	String to_String();
	
	/**
	 * Returns the type of log entry this is.
	 * @return The type of log entry this is.
	 */
	int getLogEntryType();
	
	public static final int OBJECT_ADDITION   = 1;
	public static final int OBJECT_REMOVAL    = 2;
	public static final int METHOD_INVOKATION = 3;	
	public static final int ASSERTION		  = 4;
	
}
