package edu.wpi.ejutr.logger;

/**
 * Singleton class that stores information that is input into the plugin through
 * the wizard.  The FileWriter class uses information contained here to direct
 * its output to the proper file.
 * 
 * @author Ben Mohlenhoff - <a href="mailto:bigben@wpi.edu">bigben@wpi.edu</a>
 *
 */
public class EJUTRTargetFileInfo {
	
	/**
	 * Handle to the singleton object.
	 */
	private static EJUTRTargetFileInfo me;
	
	/**
	 * Represents the container of the test file.
	 */
	private String testContainer;
	/**
	 * Represents the name of the test file.
	 */
	private String testFileName;
	/**
	 * Represents the actual test method name.
	 */
	private String testMethodName;
	
	
	/**
	 * Private constructor.
	 *
	 */
	private EJUTRTargetFileInfo() {
		me = this;
	}
	
	/**
	 * Returns the testContainer.
	 * @return the testContainer.
	 */
	public String getTestContainer() {
		return testContainer;
	}

	/**
	 * Sets the testContainer.
	 * @param testContainer the testContainer to set.
	 */
	public void setTestContainer(String testContainer) {
		this.testContainer = testContainer;
	}

	/**
	 * Gets the testFileName.
	 * @return the testFileName.
	 */
	public String getTestFileName() {
		return testFileName;
	}

	/**
	 * Sets the testFileName.
	 * @param testFileName the testFileName to set.
	 */
	public void setTestFileName(String testFileName) {
		this.testFileName = testFileName;
	}

	/**
	 * Gets the testMethodName.
	 * @return the testMethodName
	 */
	public String getTestMethodName() {
		return testMethodName;
	}

	/**
	 * Sets the testMethodName.
	 * @param testMethodName the testMethodName to set.
	 */
	public void setTestMethodName(String testMethodName) {
		this.testMethodName = testMethodName;
	}

	
	/**
	 * Returns a shared handle to the singleton object.
	 * @return the handle.
	 */
	public static EJUTRTargetFileInfo getInstance() {
		if (me == null) {
			me = new EJUTRTargetFileInfo();			
		}
		return me;
	}
	
	/**
	 * Clears all stored information from this object.
	 */
	public void clear() {
		testMethodName = null;
		testContainer = null;
		testFileName = null;
	}

}
