package edu.wpi.ejutr.logger;

/**
 * Object representing assertions made within EJUTR. The contents
 * of this object are later parsed into a string and written to a 
 * JUnit Test Method.
 * 
 * @author bigben - <a href="mailto:bigben@wpi.edu">bigben@wpi.edu</a>
 */
public class LogAssertion implements EJUTRLogEntry {

	private String lhs;
	private String rhs;
	private String comparison;
	private boolean assertResult;
	
	/**
	 * constructor.
	 * @param lhs the left-hand-side of the assertion.
	 * @param assertResult the expected result of the assertion, either true or false.
	 */
	public LogAssertion(String lhs, boolean assertResult) {
		this.lhs = lhs;
		this.assertResult = assertResult;
		rhs = "";
		comparison = "";
	}
	
	/**
	 * constructor.
	 * @param lhs the left-hand-side of the assertion.
	 * @param comparison the comparison operator used within this assertion.
	 * @param rhs the right-hand-side of the assertion. 
	 * @param assertResult the expected result of the assertion.
	 */
	public LogAssertion(String lhs, String comparison, String rhs, boolean assertResult) {
		this.lhs = lhs;
		this.comparison = comparison;
		this.rhs = rhs;
		this.assertResult = assertResult;
	}
	
	public String to_String() {
		if (assertResult) {
			return "Assert.assertTrue(" + lhs + comparison + rhs +");\n";
		} else {
			return "Assert.assertTrue(!(" + lhs + comparison + rhs + "));\n";
		}
	}

	public int getLogEntryType() {
		return EJUTRLogEntry.ASSERTION;
	}

}
