package edu.wpi.ejutr.wizards;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaConventions;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;

import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.eclipse.ui.dialogs.ResourceListSelectionDialog;

public class RecordTestMethodWizardPage extends WizardPage {

	private ISelection selection;
	
	private Text containerText;
	private Text testObjectNameText;
	private Text newTestMethodNameText;
	
	public RecordTestMethodWizardPage(ISelection selection) {
		super("wizardPage");
		setTitle("Record Test Method");
		setDescription("This wizard creates a new JUnit Test Method");
		this.selection = selection;
		//selection.
	}
	
	
	public String getContainerName() {
		return containerText.getText();
	}
	
	public String getObjectName() {
		return testObjectNameText.getText();
	}
	
	public String getTestMethodName() {
		return newTestMethodNameText.getText();
	}
	
	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
			
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 9;
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		
		Label projLabel = new Label(container, SWT.NULL);
		projLabel.setText("Folder");
		containerText = new Text(container, SWT.BORDER | SWT.SINGLE);
		containerText.setLayoutData(gd);
		containerText.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
            	dialogChanged();
            }
        });
		Button button = new Button(container, SWT.PUSH);
		button.setText("Browse...");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleBrowse();
			}
		});
		
		
		Label nameLabel = new Label(container, SWT.NULL);
		nameLabel.setText("Test Object");
		testObjectNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		testObjectNameText.setLayoutData(gd);
		testObjectNameText.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
            	dialogChanged();
            }
        });
		Button filebutton = new Button(container, SWT.PUSH);
		filebutton.setText("Browse...");
		filebutton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleFileBrowse();
			}
		});
		
		Label methodNameLabel = new Label(container, SWT.NULL);
		methodNameLabel.setText("Method Name");
		newTestMethodNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		newTestMethodNameText.setLayoutData(gd);
		newTestMethodNameText.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
            	dialogChanged();
            }
        });
		
		Label fillerLabel2 = new Label(container, SWT.NULL);
		fillerLabel2.setText("");
		Label fillerLabel3 = new Label(container, SWT.NULL);
		fillerLabel3.setText("");
		
		Label informationalLabel = new Label(container, SWT.NULL);
		informationalLabel.setText("WARNING: Clicking 'Finish' will remove all objects currently in the Object Bench.");
		
		setPageComplete(true);
		setControl(container);
	}
	
	private void dialogChanged() {

		String testName = testObjectNameText.getText();
		String testMethodName = newTestMethodNameText.getText();
		String container = containerText.getText();
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		//validate class name
		IResource testCaseNameResouce = root.findMember(new Path(container+"\\"+testName));
		//valide mathod name
		IStatus validMethodNameIdentifier = 
			JavaConventions.validateIdentifier(testMethodName);
		//validate conainter name
		IResource resource = root.findMember(new Path(container));
		if ((resource == null) || !resource.exists() || (!(resource.getType() == IResource.FOLDER) && !(resource.getType() == IResource.PROJECT))) {
			setErrorMessage("That Container does not exist");
			setPageComplete(false);
		}
		else if ((testCaseNameResouce == null) || !testCaseNameResouce.exists() || !(testCaseNameResouce.getType() == IResource.FILE)) {
			setMessage("Please choose the file you want the test method to be written to.  This file MUST extend junit.framework.testcase "); 
			setPageComplete(false);
		} else {  //file exists
			IFile file = (IFile)testCaseNameResouce;
			ICompilationUnit myCU = (ICompilationUnit) JavaCore.create(file);
			String superClass = null;
			try {
				IType[] types = myCU.getTypes();
				superClass = types[0].getSuperclassTypeSignature(); 
			} catch (JavaModelException e) {
				e.printStackTrace();
			}
			//System.out.println(superClass);
			if(superClass == null || superClass.equals("TestCase")) {
				setMessage("The specified object does not extend the type TestCase. Please choose another.");
				setPageComplete(false);
			}
			else if (!validMethodNameIdentifier.isOK()) {
				setMessage("That is not a valid method name");
				setPageComplete(false);
			} else if (testMethodName.length() < 4) {
				setPageComplete(false);
			} else if (!(testMethodName.substring(0, 4).equals("test")) || testMethodName.length() < 5) {
				//setErrorMessage("The test method name needs to begin with \"test\".");
				setMessage("Enter a method name for your test method, beginning with \"test\".");
				setPageComplete(false);
			}
			else {
				//setErrorMessage(null);
				setMessage("Press 'Finish' to continue.");
				setPageComplete(true);
			}
		}
	}
	
	private void handleFileBrowse() {
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
	
		
		IContainer container = (IContainer) root.findMember(new Path(containerText.getText()));
		//System.out.println(container.exists());
		IResource[] rs = null;
		try {
			rs = container.members();
		}
		catch (org.eclipse.core.runtime.CoreException e) {
			e.printStackTrace();
		}
		
		ResourceListSelectionDialog test = 
			new ResourceListSelectionDialog(getShell(), rs);
		test.setTitle("Test Object Selection");
		test.setMessage("Select the object that you want the test methods to be written to.  This object MUST extend junit.framework.testcase !");
		
		//test.
		test.open();
		Object [] result = test.getResult();
		if (test.getResult() != null && result.length == 1) {
			//System.out.println("debug: " + ((IResource)result[0]).getName() + "\n");
			testObjectNameText.setText(((IResource)result[0]).getName());
		}
		//ElementListSelectionDialog(getShell(), ILabelProvider renderer)
	}
	
	private void handleBrowse() {
		
		ContainerSelectionDialog dialog =
			new ContainerSelectionDialog(getShell(), null, true, "Select the folder containing the JUnit test object.");
		dialog.open();
		//ContainerSelectionDialog.
		Object[] result = dialog.getResult();
		if (result.length == 1) {
			containerText.setText(((Path) result[0]).toString());
		}
		
	}

}
